#coding: utf-8
from pika.adapters import TornadoConnection
from pika.connection import URLParameters


class RabbitBinding(object):
    def __init__(self, queue, host, callback):
        self.connection_param = URLParameters(host)
        assert callable(callback)
        self.callback = callback
        self.queue = queue
        self.connection = None
        self.channel = None

    def on_connected(self, connection):
        self.connection.channel(self.on_channel_open)

    def on_channel_open(self, channel):
        self.channel = channel
        self.channel.queue_declare(queue=self.queue, durable=True, exclusive=False, auto_delete=False,
                                   callback=self.on_queue_declared)

    def on_queue_declared(self, frame):
        self.channel.basic_consume(self.handle_delivery, queue=self.queue, no_ack=True)

    def handle_delivery(self, channel, header, method, body):
        self.callback(body)

    def connect(self):
        self.connection = TornadoConnection(self.connection_param, on_open_callback=self.on_connected)

    def run(self):
        self.connect()

    def kill_me(self, clear=False):
        if clear:
            self.channel.queue_delete(queue=self.queue)
        self.connection.close()